//===========================================================================
// Beispiel zu einer an die Physik angelehnten Simulation
//===========================================================================

//===========================================================================
// includes
//===========================================================================

// GL und GLUT Bibliotheken einbinden (je nach Betriebssystem)

#ifdef WIN32
    #include <windows.h>
    #include <GL/gl.h>
    #include <GL/glut.h>
#else
    #include <GLUT/glut.h>
#endif

#include "particle.h"

#include <stdlib.h> // rand()...
#include <math.h>


//===========================================================================
// Konstanten
//===========================================================================

const unsigned int NUM_PARTICLES = 1000;
const double       PI            = asin(1.0f) * 2.0f;


//===========================================================================
// Globale variablen
//===========================================================================

int       glutWindow = 0;
CVertex   mousePosition;
CParticle particle[NUM_PARTICLES];


//===========================================================================
// Zufallsfunktion
//===========================================================================

Coord random(Coord min, Coord max)
{
    // Gleitkommazahl zwischen min und max berechnen
    return min + Coord(rand()) / Coord(RAND_MAX) * (max-min);
}


//===========================================================================
// OpenGL Transformationen initialisieren
//===========================================================================

void transform(unsigned int width, unsigned int height)
{
    GLdouble aspect;

    // Ausgabefenstergre setzen
    glViewport(0, 0, (GLint)width, (GLint)height);

    // Verhltnis Breite zu Hhe (aspect) berechnen
    aspect = (GLdouble)width / (GLdouble)height;
    
    // Transformationen fr 3D-Koordinaten bearbeiten
    glMatrixMode(GL_MODELVIEW);
    glLoadIdentity();

    // Kameraposition setzen
    gluLookAt(0.0, 0.0, 1.0,  // von wo
              0.0, 0.0, 0.0,  // nach wo
              0.0, 1.0, 0.0); // wo ist oben?

    // Perspektivische Projektion bearbeiten
    glMatrixMode(GL_PROJECTION);
    glLoadIdentity();

    // Kameraperspektive setzen
    GLdouble fieldOfView = atan(0.5/1.0)*180.0/PI*2.0;
    gluPerspective(fieldOfView,  // ffnungswinkel vertikal (field of view)
                   aspect,       // aspect ratio setzen
                   0.001, 10.0); // nahe und weite Begrenzungsebene setzen
}


//===========================================================================
// Simulation berechnen
//===========================================================================

void update()
{
    // Erstes Partikel (Kopf) ist ein Sonderfall
    // Beschleunige auf Mauszeigerposition zu
    particle[0].accelerateTowards(mousePosition, 1.0f/32.0f);
    // Partikel bewegen
    particle[0].update(0.5f, 1.5f);

    for (unsigned int i = 1; i < NUM_PARTICLES; i++)
    {
        // Auf die Kamera zutreiben lassen
        particle[i].addForce(CVertex(0.0f, 0.0f, 1.0f/256.0f));
        // Zum vorhergehenden Punkt der Kette hin beschleunigen
        particle[i].accelerateTowards(particle[i-1].position(), 1.0f/32.0f);
        // Partikel bewegen
        particle[i].update(0.5f, 1.5f);
    }

    // Eigentlich sollten zuerst _alle_ Krfte berechnet werden
    // und dann _alle_ Partikel bewegt. Da die Partikel allerdings
    // miteinander verbunden sind, bot es sich an, die Bewegungs-
    // reaktionen abzukrzen. Sonst kme eine Mausbewegung erst
    // nach 60 Frames (1 Sekunde) 60 Kettenglieder weiter hintn an.
}


//===========================================================================
// OpenGL Zeichenfunktion
//===========================================================================

void render()
{
    // Simulation aktualisieren
    update();

    // Farbe: Wei
    glColor3f(1.0f, 1.0f, 1.0f);

    // Linie zeichnen
    glBegin(GL_LINE_STRIP);
    for (unsigned int i = 0; i < NUM_PARTICLES; i++)
    {
        // Vektor setzen
        CVertex position = particle[i].position();
        glVertex3f(position.x, position.y, position.z);
    }
    glEnd();

} // render()


//===========================================================================
// GLUT callback funktionen
//===========================================================================

void display(void)
{
    // Ausgabefenster in schwarz lschen
    glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);

    // Transformationen vor dem Zeichnen setzen
    transform(glutGet(GLUT_WINDOW_WIDTH), glutGet(GLUT_WINDOW_HEIGHT));

    // Zeichnen
    render();

    // Hintergrundpuffer anzeigen
    glutSwapBuffers();
}


void idle(void)
{
    // nochmaliges Zeichnen einleiten
    if (glutWindow != 0) glutPostRedisplay();
}


void keyboard(unsigned char key, int x, int y)
{
    // Fenster schlieen
    glutDestroyWindow(glutWindow);
    glutWindow = 0;

    // Programm beenden (etwas unsauber)
    exit(0);
}


void motion(int x, int y)
{
    // Seitenverhltnis des Fensters berechnen
    Coord aspect = Coord(glutGet(GLUT_WINDOW_WIDTH )) /
                   Coord(glutGet(GLUT_WINDOW_HEIGHT));

    // Mausposition relativ zur Kamera setzen
    mousePosition.x = (-Coord(0.5) + Coord(x) / Coord(glutGet(GLUT_WINDOW_WIDTH ))) * aspect;
    mousePosition.y = ( Coord(0.5) - Coord(y) / Coord(glutGet(GLUT_WINDOW_HEIGHT)));
    mousePosition.z = Coord(0.0);
}


//===========================================================================
// Hauptfunktion
//===========================================================================

int main(int argc, char* argv[])
{
    // GLUT initialisieren
    glutInit(&argc, argv);
    glutInitDisplayMode(GLUT_RGB | GLUT_DOUBLE | GLUT_DEPTH);

    // Fenster ffnen
    glutWindow = glutCreateWindow("wurm");

    // GLUT Funktionen fr Zeichnen, Leerlauf, Tastatur und Maus setzen
    glutDisplayFunc(display);
    glutIdleFunc(idle);
    glutKeyboardFunc(keyboard);
    glutMotionFunc(motion);
    glutPassiveMotionFunc(motion);

    // Vollbildmodus
    glutFullScreen();

    // Hauptschleife des Programms aufrufen
    glutMainLoop();

    // Bis hierhin kommt das Programm nicht
    return 0;
}
